import sys
import subprocess
from .colors import RESET
from .colors import GREEN
from .colors import RED
from .colors import BOLD


def print_default(text=''):
    sys.stdout.write(RESET)
    sys.stdout.write(f'  {text}')


def print_ok(lines='', verbose=False):
    sys.stdout.write(GREEN)
    sys.stdout.write(' OK\n')
    sys.stdout.write(RESET)
    if verbose:
        print(lines)


def print_error(lines=''):
    sys.stdout.write(RED)
    sys.stdout.write(' ERROR\n')
    sys.stdout.write(RESET)
    print(lines)


def print_header(text=''):
    sys.stdout.write(RESET)
    sys.stdout.write(BOLD)
    sys.stdout.write(f'\n{text}\n\n')


def print_empty():
    sys.stdout.write('\n')


def shell_run(cmd):
    ps = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)  # nosec
    lines = ps.communicate()[0]
    lines = lines.decode("utf-8")
    return lines



import os
from .helpers import print_header
from .helpers import print_default
from .helpers import shell_run
from .helpers import print_error
from .helpers import print_ok
from .helpers import print_empty
import datetime
from .constants import CONFIG_FILE_NAME_FLAKE8, CONFIG_FILE_CONTENT_FLAKE8
from .constants import CONFIG_FILE_NAME_RADON, CONFIG_FILE_CONTENT_RADON
from .constants import CONFIG_FILE_NAME_BANDIT, CONFIG_FILE_CONTENT_BANDIT


def create_config(directory, config_file_name, config_file_content):
    path = os.path.join(directory, config_file_name)
    if not os.path.isfile(path):
        with open(path, 'w') as f:
            f.write(config_file_content)


def create_configuration_files(directory):
    create_config(directory, CONFIG_FILE_NAME_FLAKE8, CONFIG_FILE_CONTENT_FLAKE8)
    create_config(directory, CONFIG_FILE_NAME_RADON, CONFIG_FILE_CONTENT_RADON)
    create_config(directory, CONFIG_FILE_NAME_BANDIT, CONFIG_FILE_CONTENT_BANDIT)


def run_qa(directory, verbose=False):
    #
    os.chdir(directory)
    create_configuration_files(directory)
    #
    error_count = 0
    start_at = datetime.datetime.now()
    #
    print_header('Input')
    print_default(f'Directory: {directory}\n')
    print_default(f'Start at: {start_at}\n')

    print_header('Checking')

    # flake8 for backend
    print_default(f'Checking style guide with flake8 (see "{CONFIG_FILE_NAME_FLAKE8}")')
    backend_dir = directory
    cmd = f'flake8 {backend_dir}'
    lines = shell_run(cmd)
    if lines == '':
        print_ok(lines, verbose)
    else:
        print_error(lines)
        error_count += 1

    # Unit tests
    print_default('Django unit tests')
    cmd = f'python3 {directory}/manage.py test --keepdb'
    lines = shell_run(cmd)
    if 'OK' in lines:
        print_ok(lines, verbose)
    else:
        print_error(lines)
        error_count += 1

    # Cyclomatic complexity
    print_default(f'Cyclomatic complexity with radon (see "{CONFIG_FILE_NAME_RADON}")')
    cmd = f'radon cc {directory}'
    lines = shell_run(cmd)
    if lines == '':
        print_ok(lines, verbose)
    else:
        print_error(lines)
        error_count += 1

    # Security linter
    print_default(f'Security lint with bandit (only high-severity issues, see "{CONFIG_FILE_NAME_BANDIT}")')
    lines = shell_run(f'bandit -r {directory} -lll')
    if 'No issues identified' in lines:
        print_ok(lines, verbose)
    else:
        print_error(lines)
        error_count += 1

    # *** RESULT ***
    end_at = datetime.datetime.now()
    duration = end_at - start_at

    print_header('Result')
    print_default(f'Problems found: {error_count}\n')
    print_default(f'End at: {end_at}\n')
    print_default(f'Duration: {duration}\n')
    print_empty()
    if error_count > 0:
        exit(1)



# flake8
CONFIG_FILE_NAME_FLAKE8 = '.flake8'
CONFIG_FILE_CONTENT_FLAKE8 = '''[flake8]
ignore = E501
exclude = .git,__pycache__,old,build,dist,venv,*/migrations/*,*/settings/*
max-complexity = 10
per-file-ignores = __init__.py: F401, F403
'''

# radon
CONFIG_FILE_NAME_RADON = 'radon.cfg'
CONFIG_FILE_CONTENT_RADON = '''[radon]
cc_min = C
'''

# bandit
CONFIG_FILE_NAME_BANDIT = '.bandit'
CONFIG_FILE_CONTENT_BANDIT = '''[bandit]
'''



from setuptools import setup, find_packages
from os import path

here = path.join(path.abspath(path.dirname(__file__)), 'garpix_qa')

with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='garpix_qa',
    version='1.1.1',
    description='Checking the Django project for quality',
    long_description=long_description,
    url='https://github.com/garpixcms/garpix_qa',
    author='Garpix LTD',
    author_email='info@garpix.com',
    license='MIT',
    packages=find_packages(exclude=['testproject', 'testproject.*']),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'Django >= 1.11',
        'flake8 >= 3.8.4',
        'flake8-polyfill >= 1.0.2',
        'radon >= 4.3.2',
        'bandit >= 1.7.0',
    ],
)



import os
from django.conf import settings
from django.core.management.base import BaseCommand
from garpix_qa.main import run_qa


class Command(BaseCommand):
    help = 'Checking the Django project for quality'

    def add_arguments(self, parser):
        parser.add_argument(
            '--verbose',
            action='store_true',
            help='Verbose mode',
        )

    def handle(self, *args, **options):
        directory = os.path.abspath(os.path.join(settings.BASE_DIR))
        verbose = options['verbose']
        run_qa(directory, verbose)
